<?php

namespace Drupal\visitor_analytics\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\visitor_analytics\VisitorAnalyticsRequest;

/**
 * Class LoginForm.
 */
class LoginForm extends FormBase {


  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'login_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $data = \Drupal::state()->get('visitor_analytics', []);

    $form['#images'] = drupal_get_path('module', 'visitor_analytics');
    $form['#attached']['library'][] = 'visitor_analytics/visa-styles';

    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email'),
      '#weight' => '0',
      '#default_value' => isset($data['email']) ? $data['email'] : '',
      '#required' => TRUE,
      '#title_display' => 'invisible',
      '#theme_wrappers' => [],
      '#attributes' => [
        'class' => ['notification-input'],
      ],
    ];

    $form['password'] = [
      '#type' => 'password',
      '#title' => $this->t('Password'),
      '#maxlength' => 64,
      '#weight' => '1',
      '#required' => TRUE,
      '#title_display' => 'invisible',
      '#theme_wrappers' => [],
      '#attributes' => [
        'class' => ['notification-input'],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Login'),
      '#weight' => '2',
      '#attributes' => [
        'class' => ['button', 'theme-green',],
      ],
    ];

    $form['#theme'] = 'visitor_analytics_login_form';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $data = [
      'email' => $form_state->getValue('email'),
      'password' => $form_state->getValue('password'),
    ];

    $request = new VisitorAnalyticsRequest();
    try {
      $response_data = \GuzzleHttp\json_decode($request->loginUser($data)->getBody());
    } catch (\Exception $e) {
      \Drupal::messenger()->addError($this->t($e->getMessage()));
      return ;
    }

    // save successful login data
    $visitor_analytics_data = \Drupal::state()->get('visitor_analytics', []);
    $visitor_analytics_data['token'] = $response_data->token;
    $visitor_analytics_data['refreshToken'] = $response_data->refreshToken;
    $visitor_analytics_data['userId'] = $response_data->userId;
    $visitor_analytics_data['token_created'] = time();
    $visitor_analytics_data['email'] = $form_state->getValue('email');
    \Drupal::state()->set('visitor_analytics', $visitor_analytics_data);
    $form_state->setRedirect('visitor_analytics.main_page_controller_page');
  }

}
